<!--- Deploy -->

# Deploy helm chart

## Introduction

This chart bootstraps a deployment on a [Kubernetes](https://kubernetes.io) cluster using [Helm](https://helm.sh) package manager.

## Prerequisites

The code was tested on **Kubernetes cluster** (v1.21.11) with **Istio** (1.12.6)

> It is possible to use other versions, but it hasn't been tested

### Operation system

The code works in Debian-based Linux (Debian 10 and Ubuntu 20.04) and Windows WSL 2. All other operating systems, including macOS, are not verified and supported.

### Packages

Packages are only needed for installation from a local computer.

- **HELM** (version: v3.7.1 or higher) [helm](https://helm.sh/docs/intro/install/)
- **Kubectl** (version: v1.21.0 or higher) [kubectl](https://kubernetes.io/docs/tasks/tools/#kubectl)

## Installation

First you need to set variables in **values.yaml** file using any code editor. Some of the values are prefilled, but you need to specify some values as well. You can find more information about them below.

### Global variables

| Name | Description | Type | Default |Required |
|------|-------------|------|---------|---------|
**global.domain** | your domain for the external endpoint, ex `example.com` | string | - | yes
**global.onPremEnabled** | whether on-prem is enabled | boolean | false | yes
**global.limitsEnabled** | whether CPU and memory limits are enabled | boolean | true | yes

### Configmap variables

| Name | Description | Type | Default |Required |
|------|-------------|------|---------|---------|
**data.logLevel** | logging level | string | INFO | yes
**data.googleCloudProject** | your Cloud Provider Cloud cloud project | string | -| yes
**data.entitlementsHost** | Entitlements host | string | "<http://entitlements>" | yes
**data.partitionHost** | Partition host | string | "<http://partition>" | yes
**data.recordsChangedPubsubEndpoint** | Endpoint to PubSub topic | string | - | yes
**data.storageHost** | Storage host | string | "<http://storage>" | yes
**data.secretHost** | Secret host | string | "<http://secret>" | yes
**data.serviceIdentity** | Service account in Google Cloud fo PubSub or other provider| string | "register-pubsub-identity" | yes
**data.keyRing** | Type of keys | string | "csqp" | yes
**data.kmsKey** | Key management service key (must be replaced during installation) | string | "registerService" | yes
**data.subscriberPrivateKeyId** | Subscriber test private key id (must be replaced during installation) | string | "testkey" | yes

### Deployment variables

| Name | Description | Type | Default |Required |
|------|-------------|------|---------|---------|
**data.image** | your image name | string | - | yes
**data.requestsCpu** | amount of requests CPU | string | 10m | yes
**data.requestsMemory** | amount of requests memory| string | 450Mi | yes
**data.limitsCpu** | CPU limit | string | 1.5 | only if `global.limitsEnabled` is true
**data.limitsMemory** | memory limit | string | 1G | only if `global.limitsEnabled` is true
**data.serviceAccountName** | name of your service account | string | register | yes
**data.imagePullPolicy** | when to pull image | string | IfNotPresent | yes

### Config variables

| Name | Description | Type | Default |Required |
|------|-------------|------|---------|---------|
**conf.appName** | name of the app | string | register | yes
**conf.configmap** | configmap to be used | string | register-config | yes
**conf.registerPostgresSecretName** | secret for postgres | string | register-postgres-secret | yes
**conf.registerKeycloakSecretName** | secret for keycloak | string | register-keycloak-secret | yes
**conf.registerKmsSecretName** | secret for kms | string |  "register-kms-secret" | yes
**conf.rabbitmqSecretName** | secret for rabbitmq | string | rabbitmq-secret | yes

### ISTIO variables

| Name | Description | Type | Default |Required |
|------|-------------|------|---------|---------|
**istio.proxyCPU** | CPU request for Envoy sidecars | string | 10m | yes
**istio.proxyCPULimit** | CPU limit for Envoy sidecars | string | 200m | yes
**istio.proxyMemory** | memory request for Envoy sidecars | string | 64Mi | yes
**istio.proxyMemoryLimit** | memory limit for Envoy sidecars | string | 256Mi | yes

### Install the helm chart

Run this command from within this directory:

```console
helm install core-plus-register-deploy .
```

## Uninstalling the Chart

To uninstall the helm deployment:

```console
helm uninstall core-plus-register-deploy
```

To delete secrets and PVCs:

```console
kubectl delete secret --all; kubectl delete pvc --all
```

[Move-to-Top](#deploy-helm-chart)
